
/* 
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:  
 *       "This product includes software developed by the 
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Jakarta-Regexp", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 */ 

// Java stuff
import java.util.*;
import java.io.StringWriter;
import java.io.PrintWriter;
import java.io.IOException;
import java.io.FileNotFoundException;
import java.io.FileInputStream;

// Servlet stuff
import javax.servlet.*;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

// WebStringTemplates
import au.com.forward.webstringtemplate.*;


 
public class ErrorServlet extends HttpServlet {
	
	// just one group for all threads
	static WebStringTemplateGroup errorGroup = null; 
	
	public void init() throws ServletException {
		// this is synchronized internally so OK to call in many methods on many threads.
		errorGroup = WebStringTemplateServer.getInstanceOfGroup(getServletContext(), "errorGroup",
				getServletContext().getRealPath("/") + "st");		
	}

	public void doGet(HttpServletRequest request,
			HttpServletResponse response)
			 throws ServletException, IOException {

		// get the error template from the cache
		WebStringTemplate errorTemplate = errorGroup.getInstanceOf("error",(String)null);
		
		// add the error info to the webstringtemplate
		putError(errorTemplate, "error", request);	
		//errorTemplate.setAttribute(WebStringTemplate.OUTPUT_DATA_DEFINITION_FLAG,"true");
		
		// ok now output the result
		PrintWriter out = response.getWriter();
		// then write the data to the response
		errorTemplate.write(out);
		out.close();

	}


			
	public void doPost(HttpServletRequest request, HttpServletResponse response)
			 throws ServletException, java.io.IOException {
		doGet(request, response);
	}

	/**
	 * Put the error info from the request into a map and add it to the WebStringTemplate
	 * under the name given.<br>
	 * The map holds the following entries:-
	 * status_code, exception_type, message, stackTrace, request_uri, servlet_name.
	 *
	 */
  private void putError(WebStringTemplate wst, String attributeName, ServletRequest request) {	
		// put the error info in the template under map attribute called "error"
		HashMap map = new HashMap();
		map.put("status_code", request.getAttribute("javax.servlet.error.status_code"));
		map.put("exception_type", request.getAttribute("javax.servlet.error.exception_type"));
		map.put("message", request.getAttribute("javax.servlet.error.message"));
		Throwable t = (Throwable)request.getAttribute("javax.servlet.error.exception");
		map.put("exception", t);
    StringWriter sw = new StringWriter();
    t.printStackTrace( new PrintWriter( sw ) );
		map.put("stackTrace", sw.toString());
		map.put("request_uri", request.getAttribute("javax.servlet.error.request_uri"));
		map.put("servlet_name", request.getAttribute("javax.servlet.error.servlet_name"));
		wst.put(attributeName, map);
  }	
	
	public String getServletInfo() {
		return "A WebStringTemplate version of Apache's Velocity forumdemo.";
	}
				

    /**
     *  Override the method from VelocityServlet to produce an intelligent 
     *  message to the browser
     */
    protected  void error( HttpServletRequest request, HttpServletResponse response, Exception cause )
        throws ServletException, IOException
    {
        HttpSession sess = request.getSession();
        String err = "errmsg"; //(String) sess.getAttribute( ERR_MSG_TAG );

        StringBuffer html = new StringBuffer();
        html.append("<html>");
        html.append("<body bgcolor=\"#ffffff\">");
        html.append("<h2>ForumDemo : Error processing the request</h2>");
        html.append("<br><br>There was a problem in the request." );
        html.append("<br><br>The relevant error is :<br>");
        html.append( err );
        html.append("<br><br><br>");
        html.append("The error occurred at :<br><br>");

        StringWriter sw = new StringWriter();
        cause.printStackTrace( new PrintWriter( sw ) );

        html.append( sw.toString()  );
        html.append("</body>");
        html.append("</html>");
        response.getOutputStream().print( html.toString() );
    }
}





